<?php

/**
 * Simple component for handling SFTP functionalities
 * Require PECL ssh2 installed on your environment
 *
 * @author	Aruna Attanayake <aruna470@gmail.com>
 * @version 1.0
 */

class SftpComponent extends CApplicationComponent
{
	/**
     * @var string $host sftp host ip.
     */
	public $host = null;
	 
	/**
     * @var string $port sftp host port default 22.
     */
	public $port = '22';
	
	/**
     * @var string $username username of remote sftp account.
     */
	public $username = null;
	
	/**
     * @var string $username username of remote sftp account.
     */
	public $password = null;
		
	/**
     * @var string $connection sftp connection resouce.
     */
	private $connection = null;
	
	/**
     * @var string $sftp sftp connection resouce.
     */
	private $sftp = null;
	
	/**
	 * @param string $host
	 * @param string $username
	 * @param string $password
	 * @param string $port
	 */
	public function __construct($host = null, $username = null, $password = null, $port = '22')
	{
      	$this->host = $host;
      	$this->username = $username;
     	$this->password = $password;
      	$this->port = $port;
	}
	
	/**
	 * Initializes the component.
	 */
	public function init()
	{
		parent::init();
	}
	
	/**
	 * Connect to SFTP if it is currently not
	 * @return bool true when connection success
	 * @throws CException if connection fails
	 */
	public function connect() 
	{
		if (null == $this->connection)
		{
			if ($this->connection = ssh2_connect($this->host, $this->port)) 
			{
				if (ssh2_auth_password($this->connection, $this->username, $this->password))
				{
					$this->sftp = ssh2_sftp($this->connection); 
					return true;
				}
				else
				{
					throw new CException('Invalid username or password.');
				}
			} 
		}
		else 
		{
			throw new CException('Host unreachable.');
		}
	}
	
	/**
	 * list directory contents
	 * @param string $directory Directory path
	 * @param bool $showHiddenFiles default false, if true list hidden files also
	 * @return array $files list of contents including directories
	 * @throws CException if directory open fails
	 */
	public function listFiles($directory = '', $showHiddenfiles = false)
	{
		$dir = "ssh2.sftp://{$this->sftp}/{$directory}";
		$dh = @opendir($dir);
		
		if ($dh)
		{		
			$files = array();
			
			while (false !== ($file = readdir($dh)))
			{
				if (!$showHiddenfiles && ('.' == $file || '..' == $file || '.' == $file[0]))
					continue;
				
				$files[] = $file;
			}
			
			closedir($dh);
			
			return $files;
		}
		else
		{
			throw new CException('Directory open fails.');
		}
	}

	/**
	 * Returns the current directory
	 * @return string Current directory path
	 */
	public function getCurrentDir()
	{
		$response = $this->execCmd('pwd');
		
		return trim($response);
	}

	/**
	 * Check for directory
	 * @param string $directory Directory path
	 * @return bool true if is a directory otherwise false
	 */
	public function isDir($directory)
	{
		$dir = "ssh2.sftp://{$this->sftp}/{$directory}";
		$dh = @opendir($dir);
		
		if ($dh)
		{
			closedir($dh);
			return true;
		}
		
		return false;
	}

	/**
	 * Put file to a sftp location
	 * @param string $localFile Local file path
	 * @param string $remoteFile Remote file path
	 * @param string $permission File permission default 777
	 * @return array bool true if file transfer success
	 * @throws CException if file transfer fails
	 */
	public function sendFile($localFile, $remoteFile, $permission='777')
	{
		if (@ssh2_scp_send($this->connection, $localFile, $remoteFile, $permission)) 
		{
			return true;
		} 
		else 
		{
			throw new CException('File send failed.');
		}
	}
	
	/**
	 * Get file from sftp location
	 * @param string $remoteFile Remote file path
	 * @param string $localFile Local file path
	 * @return array bool true if file retreival success
	 * @throws CException if file transfer fails
	 */
	public function getFile($remoteFile, $localFile) 
	{
		if (ssh2_scp_recv($this->connection, $remoteFile, $localFile)) 
		{
			return true;
		} 
		else 
		{
			throw new CException('File get failed.');
		}
	}
	
	/**
	 * Retreive file attributes 
	 * @param string $file Remote file path
	 * @param string $attribute Required attribute (size, gid, uid, atime, mtime, mode)
	 * @return string Attribute value
	 */
	private function getFileStat($file, $attribute)
	{
		$statinfo = ssh2_sftp_stat($this->sftp, $file);
		
		return $statinfo[$attribute];
	}
	
	/**
	 * Retreive file size 
	 * @param string $file Remote file path
	 * @return string File size
	 */
	function getSize($file)
	{
		return $this->getFileStat($file, 'size');
	}
	
	/**
	 * Retreive file modified datetime 
	 * @param string $file Remote file path
	 * @return string File modified timestamp
	 */
	function getMdtm($file)
	{
		return $this->getFileStat($file, 'mtime');
	}
	
	/**
	 * Retreive file created datetime 
	 * @param string $file Remote file path
	 * @return string File created timestamp
	 */
	function getAtime($file)
	{
		return $this->getFileStat($file, 'atime');
	}
	
	/**
	 * Create directory on sftp location
	 * @param string $directory Remote directory path
	 * @param string $permission Directory permission default 777
	 * @param bool $recursive Whether recursive directory creation
	 * @return array bool true if directory creation success
	 * @throws CException if directory creation fails
	 */
	function createDirectory($directory, $permission = 777, $recursive = true)
	{
		if (ssh2_sftp_mkdir($this->sftp, $directory, $permission, $recursive))
		{
			return true;
		}
		else
		{ 
			throw new CException('Directory creation failed.');
		}
	}
	
	/**
	 * Execute command on remote shell
	 * @param string $cmd Command ex:pwd
	 * @return string $output Command output
	 */
	function execCmd($cmd) 
	{
		$stream = ssh2_exec($this->connection, $cmd);
		stream_set_blocking($stream, true);
		
		$output = '';
		
		while ($get = fgets($stream))
		{
			$output .= $get;
		}
		
		return $output;
	}
}
?>